/*
 * FastaParser.java
 *
 * Created on Oct 4, 2007, 5:00:00 PM
 *
 * A parser for fasta files, stepping through the file by calling getFastaSequence.
 *
 */

package nl.uu.chem.bioinformatics.statquant.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import nl.uu.chem.bioinformatics.statquant.model.FastaSequence;

/**
 *
 * @author Toorn101
 */
public class FastaParser {

    private Pattern titlePattern = Pattern.compile(">([^| .]*) (.*)");
    private RandomAccessFile fastareader;
    private String filename;
    private String lineBuffer;
    private boolean waitingForSequence = true;
    private boolean uppercase = true;
    private long filelength;

    public FastaParser(File fastafile) throws FileNotFoundException {
        this.fastareader = new RandomAccessFile(fastafile, "r" );
        this.filelength = fastafile.length();
        this.filename = fastafile.getAbsolutePath();
    }

    /**
     * close function for the contained bufferedreader
     */
    public void close() throws IOException {
        if (fastareader != null) {
            fastareader.close();
        }
    }

    /**
     * Read a single concatenated sequence from the fasta file
     */
    public FastaSequence getSequence() throws IOException {
        return getSequence(null);
    }
    
    /**
     * Read a single concatenated sequence from the fasta file
     * @param readMaxLines maximum number of lines to be read before giving up, null if no check
     */
    public FastaSequence getSequence(Integer readMaxLines) throws IOException {
        String line;
        StringBuilder sequence = new StringBuilder();
        String description = "";
        String id = "NOID";
        int linecount = 0;

        if (fastareader == null) {
            throw new NullPointerException("The fastareader is not set."); //Explicit exception
        }
        FastaSequence fastasequence = null;

        if (lineBuffer == null) {
            line = fastareader.readLine();
        } else {
            line = lineBuffer;
        }
        
        while (line != null) {

            if (waitingForSequence) {
                if (line.startsWith(">")) {
                    Matcher titlematcher = titlePattern.matcher(line);
                    if (titlematcher.matches()) {
                        id = titlematcher.group(1);
                        description = titlematcher.group(2);
                        waitingForSequence = false;
                    }
                } else {
                    if (readMaxLines != null && linecount++ > readMaxLines) break;
                }
            } else {
                if (line.startsWith(">")) {
                    fastasequence = new FastaSequence(uppercase? sequence.toString().toUpperCase() : sequence.toString(), id, description);
                    waitingForSequence = true;
                    lineBuffer = line;
                    break;
                } else {
                    sequence.append(line.trim().replaceAll(" ", ""));
                }
            }

            line = fastareader.readLine();
        }
        if (id != null && sequence.toString().length() != 0) {
            fastasequence = new FastaSequence(uppercase? sequence.toString().toUpperCase() : sequence.toString(), id, description);
        }

        return fastasequence;
    }

    /**
     * Return a string representation of the regex pattern for the title
     * @return pattern string
     */
    public String getTitlePattern() {
        return titlePattern.toString();
    }

    /**
     * set the pattern to use to take apart the fasta title lines
     * @param titleParserRegex string with a regex to split the title line into two (2) groups, without the '>'
     */
    public void setTitlePattern(String titleParserRegex) {

        this.titlePattern = Pattern.compile(">" + titleParserRegex); // Saves an import in the calling package

    }

    /**
     * string representation for printing of the class, shows filename if any file is open.
     * @return string representation
     */
    @Override
    public String toString() {
        String fileinfo = "(No file open)";
        if (fastareader != null) {
            fileinfo = "(" + filename + ")";
        }
        return "FastaParser " + fileinfo;
    }

    /**
     * Is upper case mode on?
     * @return boolean for upper case mode
     */
    public boolean isUpperase() {
        return uppercase;
    }

    /**
     * Set switch to uppercase mode
     * @param onlyUppercase 
     */
    public void setUppercase(boolean uppercase) {
        this.uppercase = uppercase;
    }

    public long getFilePointer() throws IOException {
        return fastareader.getFilePointer();
    }
    
    public int getFileProgress() throws IOException {
        return (int)(fastareader.getFilePointer()*100 / filelength);
    }
}